import crypto from "crypto";
import superagent from "superagent";
import superagentAbsolute from "superagent-absolute";

export default class Khipu {
  constructor(configuration) {
    this.clientPrepare(configuration);
  }

  clientPrepare(configuration) {
    const { receiverID, secret, basePath } = configuration;
    if (!receiverID || !secret || !basePath) {
      throw new Error("The receiver ID and/or secret or basePath is not configured properly");
    }

    const agent = superagent.agent();
    const RESTClient = superagentAbsolute(agent)(basePath);

    this.callAPI = (method, path, params, sign) => {
      const { queryParams, formParams } = params || {};
      let call;

      if (method == "DELETE") {
        call = RESTClient.delete;
      } else if (method == "GET") {
        call = RESTClient.get;
      } else if (method == "HEAD") {
        call = RESTClient.head;
      } else if (method == "PATCH") {
        call = RESTClient.patch;
      } else if (method == "POST") {
        call = RESTClient.post;
      } else if (method == "PUT") {
        call = RESTClient.put;
      } else {
        throw new Error("Unknown method type");
      }

      call = call(path).set("Accept", "application/json").set("User-Agent", "KhipuNode/0.1");
      if (queryParams) {
        call = call.query(queryParams);
      } else if (formParams) {
        call = call.type("form").send(formParams);
      }

      if (sign) {
        const mergedParams = { ...queryParams, ...formParams };
        const keys = Object.keys(mergedParams).sort();

        let toSign = `${method}&${encodeURIComponent(basePath + unescape(path))}`;
        keys.forEach((key) => {
          if (mergedParams[key] === undefined) {
            return;
          }
          toSign += (`&${encodeURIComponent(key)}=${
            mergedParams[key] == null ? "" : encodeURIComponent(mergedParams[key])
              .replace(/[!'()*]/g, (c) => `%${c.charCodeAt(0).toString(16)}`)}`);
        });

        const hmac = crypto.createHmac("sha256", secret).update(toSign).digest("hex");
        call = call.set("Authorization", `${receiverID}:${hmac}`);
      }
      return call;
    };
  }

  getErrorMessage(err) {
    let message = err.response && err.response.body && err.response.body.message;
    if (err.response && err.response.body && err.response.body.errors) {
      err.response.body.errors.forEach((error) => {
        message += `\n\t${error.field}: ${error.message}`;
      });
    }
    return message;
  }

  banksGet() {
    return this.callAPI("GET", "/banks", {}, true)
      .then((response) => response.body)
      .catch((err) => {
        const message = this.getErrorMessage(err);
        throw new Error(`Cannot get banks -> ${err.status}: ${err.message}; ${message}`);
      });
  }

  paymentsGet(notification_token) {
    if (!notification_token) {
      throw new Error("Missing the required parameter notification_token when calling paymentsGet");
    }

    return this.callAPI("GET", "/payments", { queryParams: { notification_token } }, true)
      .then((response) => response.body)
      .catch((err) => {
        const message = this.getErrorMessage(err);
        throw new Error(`Cannot get payments -> ${err.status}: ${err.message}; ${message}`);
      });
  }

  paymentsPost(subject, currency, amount, options) {
    if (!subject || !currency || amount == null) {
      throw new Error("Trying to create a payment without the required parameters");
    }

    const formParams = { subject, currency, amount, ...options };

    return this.callAPI("POST", "/payments", { formParams }, true)
      .then((response) => response.body)
      .catch((err) => {
        const message = this.getErrorMessage(err);
        throw new Error(`Cannot create payment -> ${err.status}: ${err.message}; ${message}`);
      });
  }

  paymentGet(id) {
    if (!id) {
      throw new Error("Missing the required ID to get payment information");
    }

    return this.callAPI("GET", `/payments/${id}`, {}, true)
      .then((response) => response.body)
      .catch((err) => {
        const message = this.getErrorMessage(err);
        throw new Error(`Cannot get payment info -> ${err.status}: ${err.message}; ${message}`);
      });
  }

  paymentDelete(id) {
    if (!id) {
      throw new Error("Missing the required ID to remove payment information");
    }

    return this.callAPI("DELETE", `/payments/${id}`, {}, true)
      .then((response) => response.body)
      .catch((err) => {
        const message = this.getErrorMessage(err);
        throw new Error(`Cannot delete payment -> ${err.status}: ${err.message}; ${message}`);
      });
  }

  paymentConfirm(id) {
    if (!id) {
      throw new Error("Missing the required ID to confirm the payment");
    }

    return this.callAPI("POST", `/payments/${id}/confirm`, {}, true)
      .then((response) => response.body)
      .catch((err) => {
        const message = this.getErrorMessage(err);
        throw new Error(`Cannot confirm payment -> ${err.status}: ${err.message}; ${message}`);
      });
  }

  paymentRefund(id, amount) {
    if (!id) {
      throw new Error("Missing the required ID to confirm the payment");
    }
    const params = amount != null && { formParams: { amount } } || {};

    return this.callAPI("POST", `/payments/${id}/refunds`, params, true)
      .then((response) => response.body)
      .catch((err) => {
        const message = this.getErrorMessage(err);
        throw new Error(`Cannot refund payment -> ${err.status}: ${err.message}; ${message}`);
      });
  }
}
