import SimpleSchema from "simpl-schema";
import ReactionError from "@reactioncommerce/reaction-error";
import { Order as OrderSchema } from "../simpleSchemas.js";

import config from "../config.js";

const inputSchema = new SimpleSchema({
  notificationToken: {
    type: String,
    optional: true
  },
  paymentID: {
    type: String,
    optional: true
  }
});

async function cancelOrder(context, order) {
  const { shipping } = order;
  let updatedOrder;

  // We need to loop over every fulfillmentGroup
  // and then loop over every item inside group
  for (const fulfillmentGroup of shipping) {
    for (const item of fulfillmentGroup.items) {
      updatedOrder = await context.mutations.cancelOrderItem(context, {
        cancelQuantity: item.quantity,
        itemId: item._id,
        orderId: order._id,
        reason: "Order cancelled because the payment failed"
      }).catch((error) => {
        Logger.error(error);
      });
    }
  }
  return updatedOrder && updatedOrder.order;
}

function statusFromKhipu(status, detail) {
  if (status == "pending" || status == "verifying") {
    return { status: "created", orderStatus: "new" };
  } else if (detail == "reversed") {
    return { status: "partialRefund", orderStatus: "new" };
  } else if (status == "rejected" || status === "") {
    return { status: "canceled", orderStatus: "coreOrderWorkflow/canceled" };
  } else if (status == "done") {
    return { status: "completed", orderStatus: "coreOrderWorkflow/processing" };
  }
  return { status: "created", orderStatus: "new" };
}

function needsToUpdate(status, newStatus) {
  if (["refunded", "partialRefund", "canceled", "error"].includes(status) || newStatus == "created") {
    return false;
  } else if (status == "completed" && newStatus != "completed") {
    throw new Error(`Payment was marked as completed, won't change it to ${newStatus}\nDo it manually`);
  }
  return status != newStatus;
}

// Don't update canceled orders.
function orderToUpdate(status, newStatus) {
  if (status == "new" &&
        ["coreOrderWorkflow/canceled", "coreOrderWorkflow/processing"].includes(newStatus)) {
    return true;
  }
  return false;
}

/**
 * @method updateKhipuInfo
 * @summary Use this mutation to request the update of a payment
 * @param {Object} context - an object containing the per-request state
 * @param {Object} input - Necessary input. See SimpleSchema
 * @returns {Promise<Object>} Object with `order` property containing the updated order
 */
export default async function updateKhipuInfo(context, input) {
  const { KHIPU_RECEIVER_ID: configReceiverID } = config;
  inputSchema.validate(input);

  const { notificationToken, paymentID } = input;
  const { khipuClient, appEvents, collections, userId } = context;
  const { Orders } = collections;

  let paymentCall;
  if (paymentID) {
    // Admins should be able to request the update of the information
    await context.validatePermissions("reaction:legacy:orders", "read");
    paymentCall = khipuClient.paymentGet(paymentID);
  } else {
    paymentCall = khipuClient.paymentsGet(notificationToken);
  }
  const paymentInfo = await paymentCall;
  const {
    amount,
    receiver_id: receiverId,
    transaction_id: referenceId,
    payment_id: receivedId,
    status: khipuStatus,
    status_detail: statusDetail,
    expires_date: expires
  } = paymentInfo;

  const expiredDate = new Date(expires);
  const compareDate = new Date(Date.now() - 42 * 60000);
  const isExpired = (khipuStatus == "pending" || khipuStatus == "verifying") && (compareDate - expiredDate) > 0;
  const { status: newStatus, orderStatus } = statusFromKhipu(isExpired ? "rejected" : khipuStatus, statusDetail);

  if (configReceiverID != receiverId) throw new Error("Invalid receiver_id");

  // Get the refferenced order
  let order;
  if (receivedId) {
    order = await Orders.findOne({ payments: { $elemMatch: { processor: "khipu", transactionId: receivedId } } });
  } else {
    throw new Error("Malformed Khipu Payment");
  }

  // TODO: Fallback to referenceId
  // if (!order && referenceId) {
  //  order = await Orders.findOne({ referenceId });
  // }
  if (!order) throw new ReactionError("not-found", "Order not found");

  if (!order.payments || order.payments.filter((payment) => payment.status != "canceled").length != 1) {
    throw new Error("Khipu Payment Plug-In only works with one valid payment");
  }

  const modifier = {
    $set: {
      updatedAt: new Date()
    }
  };

  const updatedPayments = order.payments;
  const payment = updatedPayments.find((pmt) => pmt.transactionId === receivedId);
  if (Math.round(payment.amount) != parseInt(amount)) throw new Error("Invalid amount");

  const updatePayment = needsToUpdate(payment.status, newStatus);
  const approvePayment = updatePayment
      && ["adjustments", "created"].includes(payment.status) && newStatus == "completed";

  // Skip updating if we have no updates to make
  if (!updatePayment) return { order };
  payment.status = newStatus;
  modifier.$set.payments = updatedPayments;

  const updateOrder = orderToUpdate(order.workflow.status, orderStatus);
  if (orderStatus && updateOrder) {
    modifier.$set["workflow.status"] = orderStatus;
    modifier.$push = {
      "workflow.workflow": orderStatus
    };
  }

  OrderSchema.validate(modifier, { modifier: true });

  if (orderStatus == "coreOrderWorkflow/canceled") {
    await cancelOrder(context.getInternalContext(), order);
  }

  const { modifiedCount, value: updatedOrder } = await Orders.findOneAndUpdate(
    { _id: order._id },
    modifier,
    { returnOriginal: false }
  );
  if (modifiedCount === 0 || !updatedOrder) throw new ReactionError("server-error", "Unable to update order");

  if (orderStatus != "coreOrderWorkflow/canceled") {
    await appEvents.emit("afterOrderUpdate", {
      order: updatedOrder,
      updatedBy: userId
    });
    if (approvePayment) {
      await appEvents.emit("afterOrderApprovePayment", {
        approvedBy: userId,
        order: updatedOrder
      });
    }
  }

  return { order: updatedOrder };
}
