/* eslint camelcase: 0 */
import nock from "nock";
import mockContext from "@reactioncommerce/api-utils/tests/mockContext.js";
import Khipu from "../../libkhipu/khipu.js";

jest.mock("../config.js", () => ({ KHIPU_RECEIVER_ID: "test" }), { virtual: true });
import updateKhipuInfo from "./updateKhipuInfo.js";

mockContext.khipuClient = new Khipu({
  basePath: "https://khipu.com/api/2.0",
  receiverID: "test",
  secret: "123456"
});

mockContext.mutations.cancelOrderItem = jest.fn().mockName("cancelOrderItem").mockReturnValue(Promise.resolve({}));
mockContext.getInternalContext = jest.fn().mockName("getInternalContext").mockReturnValue(mockContext);

afterEach(() => {
  jest.clearAllMocks();
});

test("should call khipuClient.paymentsGet with the proper parameters and" +
    " return a properly formatted payment", async () => {
  const paymentData = {
    notificationToken: "ABCDEF123ABBBBBBBBBBBBBBBBBBBBB"
  };

  const khipuPaymentsGetResult = {
    amount: 123,
    receiver_id: "test",
    transaction_id: "ORDER123456ABC",
    payment_id: "0A1B2C3D",
    status: "done",
    status_detail: "payed",
    expires_date: new Date(Date.now() + 60 * 1000)
  };

  nock("https://khipu.com:443")
    .get("/api/2.0/payments")
    .query({ notification_token: "ABCDEF123ABBBBBBBBBBBBBBBBBBBBB" })
    .reply(200, khipuPaymentsGetResult);

  const now = new Date();
  const group = { items: [{ _id: "ITEM_1", quantity: 5 }, { _id: "ITEM_2", quantity: 1 }] };
  mockContext.collections.Orders.findOne.mockReturnValueOnce(Promise.resolve({
    _id: "ORDER_1",
    shipping: [group],
    shopId: "SHOP_ID",
    payments: [{
      _id: "DB_PAYMENT_ID",
      createdAt: now,
      currencyCode: "CLP",
      amount: 123,
      processor: "khipu",
      status: "created",
      transactionId: "0A1B2C3D", // khipuPaymentsGetResult.payment_id
      displayName: "Testing payment",
      method: "unknown",
      mode: "authorize",
      name: "payment_khipu",
      paymentPluginName: "PI_NAME",
      shopId: "SHOP_ID",
      transactions: []
    }],
    workflow: {
      status: "new",
      workflow: ["new"]
    }
  }));

  mockContext.collections.Orders.findOneAndUpdate.mockReturnValueOnce(Promise.resolve({
    modifiedCount: 1,
    value: {}
  }));

  const result = await updateKhipuInfo(mockContext, paymentData);
  expect(mockContext.mutations.cancelOrderItem).toHaveBeenCalledTimes(0);

  expect(mockContext.collections.Orders.findOneAndUpdate).toHaveBeenCalledWith(
    { _id: "ORDER_1" },
    {
      $set: {
        "payments": [{
          _id: "DB_PAYMENT_ID",
          createdAt: now,
          currencyCode: "CLP",
          amount: 123,
          processor: "khipu",
          status: "completed",
          transactionId: "0A1B2C3D", // khipuPaymentsGetResult.payment_id
          displayName: "Testing payment",
          method: "unknown",
          mode: "authorize",
          name: "payment_khipu",
          paymentPluginName: "PI_NAME",
          shopId: "SHOP_ID",
          transactions: []
        }],
        "workflow.status": "coreOrderWorkflow/processing",
        "updatedAt": jasmine.any(Date)
      },
      $push: {
        "workflow.workflow": "coreOrderWorkflow/processing"
      }
    },
    { returnOriginal: false }
  );
});

test("should update payment with a given paymentID", async () => {
  const paymentData = {
    paymentID: "1ABCDEF123ABBBBBBBBBBBBBBBBBBBBB"
  };

  const khipuPaymentsGetResult = {
    amount: 123,
    receiver_id: "test",
    transaction_id: "ORDER123456ABC",
    payment_id: "0A1B2C3D",
    status: "done",
    status_detail: "payed"
  };

  nock("https://khipu.com:443")
    .get("/api/2.0/payments/1ABCDEF123ABBBBBBBBBBBBBBBBBBBBB")
    .reply(200, khipuPaymentsGetResult);

  const now = new Date();
  const group = { items: [{ _id: "ITEM_1", quantity: 5 }, { _id: "ITEM_2", quantity: 1 }] };
  mockContext.collections.Orders.findOne.mockReturnValueOnce(Promise.resolve({
    _id: "ORDER_1",
    shipping: [group],
    shopId: "SHOP_ID",
    payments: [{
      _id: "DB_PAYMENT_ID",
      createdAt: now,
      currencyCode: "CLP",
      amount: 123,
      processor: "khipu",
      status: "created",
      transactionId: "0A1B2C3D", // khipuPaymentsGetResult.payment_id
      displayName: "Testing payment",
      method: "unknown",
      mode: "authorize",
      name: "payment_khipu",
      paymentPluginName: "PI_NAME",
      shopId: "SHOP_ID",
      transactions: []
    }],
    workflow: {
      status: "new",
      workflow: ["new"]
    }
  }));

  mockContext.collections.Orders.findOneAndUpdate.mockReturnValueOnce(Promise.resolve({
    modifiedCount: 1,
    value: {}
  }));

  const result = await updateKhipuInfo(mockContext, paymentData);
  expect(mockContext.mutations.cancelOrderItem).toHaveBeenCalledTimes(0);

  expect(mockContext.collections.Orders.findOneAndUpdate).toHaveBeenCalledWith(
    { _id: "ORDER_1" },
    {
      $set: {
        "payments": [{
          _id: "DB_PAYMENT_ID",
          createdAt: now,
          currencyCode: "CLP",
          amount: 123,
          processor: "khipu",
          status: "completed",
          transactionId: "0A1B2C3D", // khipuPaymentsGetResult.payment_id
          displayName: "Testing payment",
          method: "unknown",
          mode: "authorize",
          name: "payment_khipu",
          paymentPluginName: "PI_NAME",
          shopId: "SHOP_ID",
          transactions: []
        }],
        "workflow.status": "coreOrderWorkflow/processing",
        "updatedAt": jasmine.any(Date)
      },
      $push: {
        "workflow.workflow": "coreOrderWorkflow/processing"
      }
    },
    { returnOriginal: false }
  );
});

test("should cancel the order on a canceled payment", async () => {
  const paymentData = {
    notificationToken: "ABCDEF123ABBBBBBBBBBBBBBBBBBBBB"
  };

  const khipuPaymentsGetResult = {
    amount: 123,
    receiver_id: "test",
    transaction_id: "ORDER123456ABC",
    payment_id: "0A1B2C3D",
    status: "rejected",
    status_detail: "pending"
  };

  nock("https://khipu.com:443")
    .get("/api/2.0/payments")
    .query({ notification_token: "ABCDEF123ABBBBBBBBBBBBBBBBBBBBB" })
    .reply(200, khipuPaymentsGetResult);

  const now = new Date();
  const group = { items: [{ _id: "ITEM_1", quantity: 5 }, { _id: "ITEM_2", quantity: 1 }] };
  mockContext.collections.Orders.findOne.mockReturnValueOnce(Promise.resolve({
    _id: "ORDER_1",
    shipping: [group],
    shopId: "SHOP_ID",
    payments: [{
      _id: "DB_PAYMENT_ID",
      createdAt: now,
      currencyCode: "CLP",
      amount: 123,
      processor: "khipu",
      status: "created",
      transactionId: "0A1B2C3D", // khipuPaymentsGetResult.payment_id
      displayName: "Testing payment",
      method: "unknown",
      mode: "authorize",
      name: "payment_khipu",
      paymentPluginName: "PI_NAME",
      shopId: "SHOP_ID",
      transactions: []
    }],
    workflow: {
      status: "new",
      workflow: ["new"]
    }
  }));

  mockContext.collections.Orders.findOneAndUpdate.mockReturnValueOnce(Promise.resolve({
    modifiedCount: 1,
    value: {}
  }));

  const result = await updateKhipuInfo(mockContext, paymentData);
  expect(mockContext.mutations.cancelOrderItem).toHaveBeenCalledTimes(2);
  expect(mockContext.mutations.cancelOrderItem).toHaveBeenCalledWith(mockContext, {
    cancelQuantity: 5,
    itemId: "ITEM_1",
    orderId: "ORDER_1",
    reason: "Order cancelled because the payment failed"
  });
  expect(mockContext.mutations.cancelOrderItem).toHaveBeenCalledWith(mockContext, {
    cancelQuantity: 1,
    itemId: "ITEM_2",
    orderId: "ORDER_1",
    reason: "Order cancelled because the payment failed"
  });

  expect(mockContext.collections.Orders.findOneAndUpdate).toHaveBeenCalledWith(
    { _id: "ORDER_1" },
    {
      $set: {
        "payments": [{
          _id: "DB_PAYMENT_ID",
          createdAt: now,
          currencyCode: "CLP",
          amount: 123,
          processor: "khipu",
          status: "canceled",
          transactionId: "0A1B2C3D", // khipuPaymentsGetResult.payment_id
          displayName: "Testing payment",
          method: "unknown",
          mode: "authorize",
          name: "payment_khipu",
          paymentPluginName: "PI_NAME",
          shopId: "SHOP_ID",
          transactions: []
        }],
        "workflow.status": "coreOrderWorkflow/canceled",
        "updatedAt": jasmine.any(Date)
      },
      $push: {
        "workflow.workflow": "coreOrderWorkflow/canceled"
      }
    },
    { returnOriginal: false }
  );
});

test("should not accept a payment after expiration date", async () => {
  const paymentData = {
    notificationToken: "ABCDEF123ABBBBBBBBBBBBBBBBBBBBB"
  };

  const khipuPaymentsGetResult = {
    amount: 123,
    receiver_id: "test",
    transaction_id: "ORDER123456ABC",
    payment_id: "0A1B2C3D",
    status: "verifying",
    status_detail: "payed",
    expires_date: new Date(Date.now() - 60 * 60 * 1000)
  };

  nock("https://khipu.com:443")
    .get("/api/2.0/payments")
    .query({ notification_token: "ABCDEF123ABBBBBBBBBBBBBBBBBBBBB" })
    .reply(200, khipuPaymentsGetResult);

  const now = new Date();
  const group = { items: [{ _id: "ITEM_1", quantity: 5 }, { _id: "ITEM_2", quantity: 1 }] };
  mockContext.collections.Orders.findOne.mockReturnValueOnce(Promise.resolve({
    _id: "ORDER_1",
    shipping: [group],
    shopId: "SHOP_ID",
    payments: [{
      _id: "DB_PAYMENT_ID",
      createdAt: now,
      currencyCode: "CLP",
      amount: 123,
      processor: "khipu",
      status: "created",
      transactionId: "0A1B2C3D", // khipuPaymentsGetResult.payment_id
      displayName: "Testing payment",
      method: "unknown",
      mode: "authorize",
      name: "payment_khipu",
      paymentPluginName: "PI_NAME",
      shopId: "SHOP_ID",
      transactions: []
    }],
    workflow: {
      status: "new",
      workflow: ["new"]
    }
  }));

  mockContext.collections.Orders.findOneAndUpdate.mockReturnValueOnce(Promise.resolve({
    modifiedCount: 1,
    value: {}
  }));

  const result = await updateKhipuInfo(mockContext, paymentData);
  expect(mockContext.mutations.cancelOrderItem).toHaveBeenCalledTimes(2);
  expect(mockContext.mutations.cancelOrderItem).toHaveBeenCalledWith(mockContext, {
    cancelQuantity: 5,
    itemId: "ITEM_1",
    orderId: "ORDER_1",
    reason: "Order cancelled because the payment failed"
  });
  expect(mockContext.mutations.cancelOrderItem).toHaveBeenCalledWith(mockContext, {
    cancelQuantity: 1,
    itemId: "ITEM_2",
    orderId: "ORDER_1",
    reason: "Order cancelled because the payment failed"
  });

  expect(mockContext.collections.Orders.findOneAndUpdate).toHaveBeenCalledWith(
    { _id: "ORDER_1" },
    {
      $set: {
        "payments": [{
          _id: "DB_PAYMENT_ID",
          createdAt: now,
          currencyCode: "CLP",
          amount: 123,
          processor: "khipu",
          status: "canceled",
          transactionId: "0A1B2C3D", // khipuPaymentsGetResult.payment_id
          displayName: "Testing payment",
          method: "unknown",
          mode: "authorize",
          name: "payment_khipu",
          paymentPluginName: "PI_NAME",
          shopId: "SHOP_ID",
          transactions: []
        }],
        "workflow.status": "coreOrderWorkflow/canceled",
        "updatedAt": jasmine.any(Date)
      },
      $push: {
        "workflow.workflow": "coreOrderWorkflow/canceled"
      }
    },
    { returnOriginal: false }
  );
});

test("should not cancel an already accepted payment", async () => {
  const paymentData = {
    notificationToken: "ABCDEF123ABBBBBBBBBBBBBBBBBBBBB"
  };

  const khipuPaymentsGetResult = {
    amount: 123,
    receiver_id: "test",
    transaction_id: "ORDER123456ABC",
    payment_id: "0A1B2C3D",
    status: "rejected",
    status_detail: "pending",
    expires_date: new Date(Date.now() - 60 * 60 * 1000)
  };

  nock("https://khipu.com:443")
    .get("/api/2.0/payments")
    .query({ notification_token: "ABCDEF123ABBBBBBBBBBBBBBBBBBBBB" })
    .reply(200, khipuPaymentsGetResult);

  const now = new Date();
  const group = { items: [{ _id: "ITEM_1", quantity: 5 }, { _id: "ITEM_2", quantity: 1 }] };
  mockContext.collections.Orders.findOne.mockReturnValueOnce(Promise.resolve({
    _id: "ORDER_1",
    shipping: [group],
    shopId: "SHOP_ID",
    payments: [{
      _id: "DB_PAYMENT_ID",
      createdAt: now,
      currencyCode: "CLP",
      amount: 123,
      processor: "khipu",
      status: "completed",
      transactionId: "0A1B2C3D", // khipuPaymentsGetResult.payment_id
      displayName: "Testing payment",
      method: "unknown",
      mode: "authorize",
      name: "payment_khipu",
      paymentPluginName: "PI_NAME",
      shopId: "SHOP_ID",
      transactions: []
    }],
    workflow: {
      status: "coreOrderWorkflow/processing",
      workflow: ["new", "coreOrderWorkflow/processing"]
    }
  }));

  mockContext.collections.Orders.findOneAndUpdate.mockReturnValueOnce(Promise.resolve({
    modifiedCount: 1,
    value: {}
  }));

  const result = updateKhipuInfo(mockContext, paymentData);
  await expect(result).rejects.toThrow(Error);
  expect(mockContext.mutations.cancelOrderItem).toHaveBeenCalledTimes(0);
  expect(mockContext.collections.Orders.findOneAndUpdate).toHaveBeenCalledTimes(0);
});
