import ReactionError from "@reactioncommerce/reaction-error";

/**
 * @name  khipuRefunds
 * @method
 * @memberof Khipu/Queries
 * @summary Query the  ReactionPaymentKhipuRefunds collection for refunds and (optionally) shopIds
 * @param {Object} context - an object containing the per-request state
 * @param {Object} params - request parameters
 * @param {Object} params.filters - Filters to apply to a list of refunds
 * @param {Array.<String>} params.shopIds - Shop IDs for the shops that created the refunds
 * @returns {Promise<Object>|undefined} - An Array of ReactionPaymentKhipuRefunds documents, if found
 */
export default async function khipuRefunds(context, { filters, shopIds } = {}) {
  const { collections } = context;
  const { ReactionPaymentKhipuRefunds } = collections;

  const query = {};
  let createdAtFilter = {};
  let statusFilter = {};

  // Validate user has permission to view orders for all shopIds
  if (!shopIds) throw new ReactionError("invalid-param", "You must provide ShopId(s)");
  for (const shopId of shopIds) {
    await context.validatePermissions("reaction:legacy:orders", "read", { shopId }); // eslint-disable-line no-await-in-loop
  }

  // If no specific shopIds where given, show result for all shops.
  if (shopIds.length == 0) {
    await context.validatePermissions("reaction:legacy:orders", "read", { shopId: null });
  } else {
    query.shopId = { $in: shopIds };
  }

  // Add a date range filter if provided, the filter will be
  // applied to the createdAt database field.
  if (filters && filters.createdAt) {
    const { createdAt } = filters;
    // Both fields are optional
    const gteProp = createdAt.gte ? { $gte: createdAt.gte } : {};
    const lteProp = createdAt.lte ? { $lte: createdAt.lte } : {};
    createdAtFilter = {
      createdAt: {
        ...gteProp,
        ...lteProp
      }
    };
  }

  // Add refund status filter if provided
  if (filters && filters.status) {
    statusFilter = {
      status: { $eq: filters.status }
    };
  }

  // Build the final query
  query.$and = [{
    ...createdAtFilter,
    ...statusFilter
  }];

  return ReactionPaymentKhipuRefunds.find(query);
}
