import Logger from "@reactioncommerce/logger";
import ReactionError from "@reactioncommerce/reaction-error";

async function changePaymentStatus(Orders, orderId, updatedPayments) {
  const modifier = {
    $set: {
      updatedAt: new Date(),
      payments: updatedPayments
    }
  };

  try {
    await Orders.findOneAndUpdate(
      { _id: orderId },
      modifier,
      { returnOriginal: false }
    );
  } catch {
    Logger.error(`Could not update payment info for order ${orderId} `);
  }
}

/**
 * @name reactionPaymentKhipuCreateRefund
 * @method
 * @summary Create a refund for an order for the Khipu payment method
 * @param {Object} context an object containing the per-request state
 * @param {Object} payment object containing transaction ID
 * @param {Number} amount the amount to be refunded
 * @param {String} [reason] the reason for the refund
 * @returns {Object} refund result
 * @private
 */
export default async function reactionPaymentKhipuCreateRefund(context, payment, amount, reason, extra) {
  const { currencyCode, transactionId } = payment;
  const { khipuClient, collections } = context;
  const { Orders } = collections;
  const { shopId } = extra || {};

  let status = "created";
  const order = await Orders.findOne({ payments: { $elemMatch: { processor: "khipu", transactionId } } });
  const updatedPayments = order.payments;
  const savedPayment = updatedPayments.find((pmt) => pmt.transactionId === transactionId);

  if (savedPayment.status == "created") {
    // try to cancel payment if the refund amount is the same as the payment amount
    if (Math.round(savedPayment.amount) === Math.round(amount)) {
      try {
        await khipuClient.paymentDelete(transactionId);
        status = "completed";

        savedPayment.status = "canceled";
        await changePaymentStatus(Orders, order._id, updatedPayments);
      } catch {
        Logger.info(`Failed to cancel the ${transactionId} payment via the KHIPU API`);
      }
    } else {
      Logger.info("Can't cancel payment for order with multiple groups");
    }
  }

  if (status != "completed") {
    try {
      // try to create a refund with khipu
      await khipuClient.paymentRefund(transactionId, Math.round(amount));
      status = "completed";

      savedPayment.status = "partialRefund";
      await changePaymentStatus(Orders, order._id, updatedPayments);
    } catch {
      if (savedPayment.status == "created") {
        // there is nothing more we can try.
        throw new ReactionError("server-error", "Can't refund not completed payment");
      }
      // if we cant create a refund with khipu, it will be saved with "created" status (to be manually transfered back)
      Logger.info("No refund available via the KHIPU API");
    }
  }

  const refund = await context.collections.ReactionPaymentKhipuRefunds.insertOne({
    amount,
    createdAt: new Date(),
    currencyCode,
    reason,
    transactionId,
    shopId,
    status
  });
  return { refundId: refund._id, saved: true };
}
