/* eslint camelcase: 0 */
import nock from "nock";
import mockContext from "@reactioncommerce/api-utils/tests/mockContext.js";
import mockCollection from "@reactioncommerce/api-utils/tests/mockCollection.js";
import Khipu from "../../libkhipu/khipu.js";

import reactionPaymentKhipuCreateRefund from "./reactionPaymentKhipuCreateRefund.js";

mockContext.collections.ReactionPaymentKhipuRefunds = mockCollection("ReactionPaymentKhipuRefunds");
mockContext.collections.ReactionPaymentKhipuRefunds.insertOne.mockReturnValue(Promise.resolve({}));

mockContext.khipuClient = new Khipu({
  basePath: "https://khipu.com/api/2.0",
  receiverID: "test",
  secret: "SECRET"
});

afterEach(() => {
  jest.clearAllMocks();
});

test("should cancel the payment on full refunds", async () => {
  const now = new Date();
  const paymentData = {
    _id: "DB_PAYMENT_ID",
    createdAt: now,
    currencyCode: "CLP",
    amount: 123,
    processor: "khipu",
    status: "created",
    transactionId: "0A1B2C3D",
    displayName: "Testing payment",
    method: "unknown",
    mode: "authorize",
    name: "payment_khipu",
    paymentPluginName: "PI_NAME",
    shopId: "SHOP_ID",
    transactions: []
  };

  nock("https://khipu.com:443")
    .delete("/api/2.0/payments/0A1B2C3D")
    .reply(200, { message: "TEST MESSAGE" });

  mockContext.collections.Orders.findOne.mockReturnValueOnce(Promise.resolve({
    _id: "ORDER_1",
    shipping: [],
    shopId: "SHOP_ID",
    payments: [{ ...paymentData }],
    workflow: {
      status: "new",
      workflow: ["new"]
    }
  }));

  const result = await reactionPaymentKhipuCreateRefund(mockContext, paymentData, 123, "Order has been cancelled", { shopId: "SHOP_ID" });
  expect(mockContext.collections.Orders.findOneAndUpdate).toHaveBeenCalledWith(
    { _id: "ORDER_1" },
    {
      $set: {
        payments: [{
          _id: "DB_PAYMENT_ID",
          createdAt: now,
          currencyCode: "CLP",
          amount: 123,
          processor: "khipu",
          status: "canceled",
          transactionId: "0A1B2C3D",
          displayName: "Testing payment",
          method: "unknown",
          mode: "authorize",
          name: "payment_khipu",
          paymentPluginName: "PI_NAME",
          shopId: "SHOP_ID",
          transactions: []
        }],
        updatedAt: jasmine.any(Date)
      }
    },
    { returnOriginal: false }
  );
  expect(mockContext.collections.ReactionPaymentKhipuRefunds.insertOne).toHaveBeenCalledWith({
    amount: 123,
    createdAt: jasmine.any(Date),
    currencyCode: "CLP",
    reason: "Order has been cancelled",
    transactionId: "0A1B2C3D",
    shopId: "SHOP_ID",
    status: "completed"
  });
});

test("should refund payment on delete failure", async () => {
  const now = new Date();
  const paymentData = {
    _id: "DB_PAYMENT_ID",
    createdAt: now,
    currencyCode: "CLP",
    amount: 123,
    processor: "khipu",
    status: "created",
    transactionId: "0A1B2C3D",
    displayName: "Testing payment",
    method: "unknown",
    mode: "authorize",
    name: "payment_khipu",
    paymentPluginName: "PI_NAME",
    shopId: "SHOP_ID",
    transactions: []
  };

  nock("https://khipu.com:443")
    .delete("/api/2.0/payments/0A1B2C3D")
    .reply(503, { message: "TEST MESSAGE" });
  nock("https://khipu.com:443")
    .post("/api/2.0/payments/0A1B2C3D/refunds", (body) => body.amount === "123")
    .reply(200, { message: "TEST MESSAGE" });

  mockContext.collections.Orders.findOne.mockReturnValueOnce(Promise.resolve({
    _id: "ORDER_1",
    shipping: [],
    shopId: "SHOP_ID",
    payments: [{ ...paymentData }],
    workflow: {
      status: "new",
      workflow: ["new"]
    }
  }));

  const result = await reactionPaymentKhipuCreateRefund(mockContext, paymentData, 123, "Order has been cancelled", { shopId: "SHOP_ID" });
  expect(mockContext.collections.Orders.findOneAndUpdate).toHaveBeenCalledWith(
    { _id: "ORDER_1" },
    {
      $set: {
        payments: [{
          _id: "DB_PAYMENT_ID",
          createdAt: now,
          currencyCode: "CLP",
          amount: 123,
          processor: "khipu",
          status: "partialRefund",
          transactionId: "0A1B2C3D",
          displayName: "Testing payment",
          method: "unknown",
          mode: "authorize",
          name: "payment_khipu",
          paymentPluginName: "PI_NAME",
          shopId: "SHOP_ID",
          transactions: []
        }],
        updatedAt: jasmine.any(Date)
      }
    },
    { returnOriginal: false }
  );
  expect(mockContext.collections.ReactionPaymentKhipuRefunds.insertOne).toHaveBeenCalledWith({
    amount: 123,
    createdAt: jasmine.any(Date),
    currencyCode: "CLP",
    reason: "Order has been cancelled",
    transactionId: "0A1B2C3D",
    shopId: "SHOP_ID",
    status: "completed"
  });
});

test("should fail if a non-completed payment can't be deleted", async () => {
  const now = new Date();
  const paymentData = {
    _id: "DB_PAYMENT_ID",
    createdAt: now,
    currencyCode: "CLP",
    amount: 123,
    processor: "khipu",
    status: "created",
    transactionId: "0A1B2C3D",
    displayName: "Testing payment",
    method: "unknown",
    mode: "authorize",
    name: "payment_khipu",
    paymentPluginName: "PI_NAME",
    shopId: "SHOP_ID",
    transactions: []
  };

  nock("https://khipu.com:443")
    .post("/api/2.0/payments/0A1B2C3D/refunds", (body) => body.amount === "123")
    .reply(503, { message: "TEST MESSAGE" });

  mockContext.collections.Orders.findOne.mockReturnValueOnce(Promise.resolve({
    _id: "ORDER_1",
    shipping: [],
    shopId: "SHOP_ID",
    payments: [{ ...paymentData }],
    workflow: {
      status: "new",
      workflow: ["new"]
    }
  }));

  const result = reactionPaymentKhipuCreateRefund(mockContext, paymentData, 123, "Order has been cancelled", { shopId: "SHOP_ID" });
  await expect(result).rejects.toThrow(Error);
  expect(mockContext.collections.Orders.findOneAndUpdate).toHaveBeenCalledTimes(0);
  expect(mockContext.collections.ReactionPaymentKhipuRefunds.insertOne).toHaveBeenCalledTimes(0);
});

test("should create local refund for completedon API failure", async () => {
  const now = new Date();
  const paymentData = {
    _id: "DB_PAYMENT_ID",
    createdAt: now,
    currencyCode: "CLP",
    amount: 123,
    processor: "khipu",
    status: "completed",
    transactionId: "0A1B2C3D",
    displayName: "Testing payment",
    method: "unknown",
    mode: "authorize",
    name: "payment_khipu",
    paymentPluginName: "PI_NAME",
    shopId: "SHOP_ID",
    transactions: []
  };

  nock("https://khipu.com:443")
    .delete("/api/2.0/payments/0A1B2C3D")
    .reply(503, { message: "TEST MESSAGE" });
  nock("https://khipu.com:443")
    .post("/api/2.0/payments/0A1B2C3D/refunds", (body) => body.amount === "123")
    .reply(503, { message: "TEST MESSAGE" });

  mockContext.collections.Orders.findOne.mockReturnValueOnce(Promise.resolve({
    _id: "ORDER_1",
    shipping: [],
    shopId: "SHOP_ID",
    payments: [{ ...paymentData }],
    workflow: {
      status: "coreOrderWorkflow/processing",
      workflow: ["new", "coreOrderWorkflow/processing"]
    }
  }));

  const result = await reactionPaymentKhipuCreateRefund(mockContext, paymentData, 123, "Order has been cancelled", { shopId: "SHOP_ID" });
  expect(mockContext.collections.Orders.findOneAndUpdate).toHaveBeenCalledTimes(0);
  expect(mockContext.collections.ReactionPaymentKhipuRefunds.insertOne).toHaveBeenCalledWith({
    amount: 123,
    createdAt: jasmine.any(Date),
    currencyCode: "CLP",
    reason: "Order has been cancelled",
    transactionId: "0A1B2C3D",
    shopId: "SHOP_ID",
    status: "created"
  });
});

test("should make partial refunds when the amount its not the total", async () => {
  const now = new Date();
  const paymentData = {
    _id: "DB_PAYMENT_ID",
    createdAt: now,
    currencyCode: "CLP",
    amount: 123,
    processor: "khipu",
    status: "completed",
    transactionId: "0A1B2C3D",
    displayName: "Testing payment",
    method: "unknown",
    mode: "authorize",
    name: "payment_khipu",
    paymentPluginName: "PI_NAME",
    shopId: "SHOP_ID",
    transactions: []
  };

  nock("https://khipu.com:443")
    .post("/api/2.0/payments/0A1B2C3D/refunds", (body) => body.amount === "50")
    .reply(200, { message: "TEST MESSAGE" });

  mockContext.collections.Orders.findOne.mockReturnValueOnce(Promise.resolve({
    _id: "ORDER_1",
    shipping: [],
    shopId: "SHOP_ID",
    payments: [{ ...paymentData }],
    workflow: {
      status: "coreOrderWorkflow/processing",
      workflow: ["new", "coreOrderWorkflow/processing"]
    }
  }));

  const result = await reactionPaymentKhipuCreateRefund(mockContext, paymentData, 50, "Order has been cancelled", { shopId: "SHOP_ID" });
  expect(mockContext.collections.Orders.findOneAndUpdate).toHaveBeenCalledWith(
    { _id: "ORDER_1" },
    {
      $set: {
        payments: [{
          _id: "DB_PAYMENT_ID",
          createdAt: now,
          currencyCode: "CLP",
          amount: 123,
          processor: "khipu",
          status: "partialRefund",
          transactionId: "0A1B2C3D",
          displayName: "Testing payment",
          method: "unknown",
          mode: "authorize",
          name: "payment_khipu",
          paymentPluginName: "PI_NAME",
          shopId: "SHOP_ID",
          transactions: []
        }],
        updatedAt: jasmine.any(Date)
      }
    },
    { returnOriginal: false }
  );
  expect(mockContext.collections.ReactionPaymentKhipuRefunds.insertOne).toHaveBeenCalledWith({
    amount: 50,
    createdAt: jasmine.any(Date),
    currencyCode: "CLP",
    reason: "Order has been cancelled",
    transactionId: "0A1B2C3D",
    shopId: "SHOP_ID",
    status: "completed"
  });
});
